﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CREAD.Framework.Utility.Bmap
{
    /// <summary>
    /// Class to generate a static map using the baidu StaticMaps API
    /// http://developer.baidu.com/map/staticimg.htm
    /// </summary>
    public class StaticMapHelper
    {
        private const string BaiduStaticMapUrl = "http://api.map.baidu.com/staticimage";

        /// <summary>
        /// Renders an image for display
        /// </summary>
        /// <returns>Primarily this just creates an imageUrl string</returns>
        public string Render()
        {
            string qs = BaiduStaticMapUrl + "?center={0},{1}&zoom={2}&weight={3}&height={4}";
            string mkqs = "";
            string mkstyles = "";


            qs = string.Format(qs, Latitude, Longitude, Zoom, Width, Height);
            //add markers
            foreach (var marker in _markers)
            {
                mkqs += string.Format("{0},{1}|", marker.Latitude, marker.Longitude);
                mkstyles += string.Format("{0}|",
                    GetMarkerParams(marker.Size, marker.Color, marker.Label));
            }
            if (mkqs.Length > 0)
            {
                qs += "&markers=" + mkqs.TrimEnd('|') + "&markerStyle=" + mkstyles.TrimEnd('|');
            }

            string mlqs = "";
            string mlstyles = "";
            //add labels
            foreach (var label in _labels)
            {
                mlqs += string.Format("{0},{1}|", label.Latitude, label.Longitude);
                mlstyles += string.Format("{0}|", GetLablesParms(label.Content, label.Border, label.FontSize,
                    label.FontColor, label.BgColor, label.FontWeight));
            }
            if (mlqs.Length > 0)
            {
                qs += "&labels=" + mlqs.TrimEnd('|') + "&labelStyles=" + mlstyles.TrimEnd('|');
            }
            return qs;
        }

        /// <summary>
        /// Build the correct string for marker parameters
        /// </summary>
        /// <param name="size"></param>
        /// <param name="color"></param>
        /// <param name="label"></param>
        /// <returns></returns>
        private string GetMarkerParams(mSize size, mColor color, string label)
        {
            string marker;

            marker = size.ToString().ToLower() + "," + color.ToString().ToLower() + "," + label;
            return marker;
        }
        //http://api.map.baidu.com/staticimage?
        //width=400&height=200&center=116.403874,39.914888&
        //markers=百度大厦|116.403874,39.914888&zoom=15&markerStyles=l,A,0xff0000

        /// <summary>
        /// Build the correct string for lable parameters
        /// </summary>
        /// <param name="content"></param>
        /// <param name="fSize"></param>
        /// <param name="fColor"></param>
        /// <param name="bgColor"></param>
        /// <returns></returns>
        private string GetLablesParms(string content, int border, int fSize,
            System.Drawing.Color fColor, System.Drawing.Color bgColor, int fontWeight)
        {
            string lables;
            lables = string.Format("{0},{1},{2},{3:x6},{4:x6},{5}", content, border, fSize,
                     fColor.ToArgb(), bgColor.ToArgb(), fontWeight);
            return lables;


        }

        //http://api.map.baidu.com/staticimage?
        //width=400&height=200&center=北京&labels=海淀|116.487812,40.017524&
        //labelStyles=小吃,1,14,0xffffff,0x000fff,1
        public StaticMapHelper()
        { }

        public StaticMapHelper(double Latitude, double Longitude, int Zoom)
        {

        }

        /// <summary>
        /// Defines a single map point to be added to a map
        /// </summary>
        public class MapMarker
        {
            private string label = "";

            public string Label
            {
                get { return label; }
                set { label = value; }
            }

            #region Auto-Properties
            public double Latitude { get; set; }
            public double Longitude { get; set; }
            public mSize Size { get; set; }
            public mColor Color { get; set; }

            public string IconUrl { get; set; }

            #endregion
        }

        /// <summary>
        /// Defines a single map label to be added to a map
        /// </summary>
        public class MapLabel
        {
            public string Content { get; set; }
            public double Latitude { get; set; }
            public double Longitude { get; set; }
            public int Border { get; set; }
            public int FontSize { get; set; }
            public System.Drawing.Color FontColor { get; set; }
            public System.Drawing.Color BgColor { get; set; }
            public int FontWeight { get; set; }
        }

        /// <summary>
        /// All Map rendering properties as enums
        /// </summary>
        #region Marker enums

        public enum mFormat
        {
            gif = 0,
            jpg = 1,
            png = 2
        }

        public enum mSize
        {
            s = 1,
            m = 2,
            l = 3
        }

        public enum mColor
        {
            Black = 0,
            Brown = 1,
            Green = 2,
            Purple = 3,
            Yellow = 4,
            Blue = 5,
            Gray = 6,
            Orange = 7,
            Red = 8,
            White = 9
        }



        #endregion


        #region Properties
        private List<MapMarker> _markers = new List<MapMarker>();
        private List<MapLabel> _labels = new List<MapLabel>();

        public List<MapMarker> Markers
        {
            get { return _markers; }
            set { _markers = value; }
        }

        public List<MapLabel> Labels
        {
            get { return _labels; }
            set { _labels = value; }
        }


        #endregion


        #region Auto-Properties
        public int Width { get; set; }
        public int Height { get; set; }
        public double Latitude { get; set; }
        public double Longitude { get; set; }
        public int Zoom { get; set; }
        public int Scale { get; set; }
        #endregion
    }
}
